@@ -15,7 +15,7 @@
    | Author: Rasmus Lerdorf                                               |
    +----------------------------------------------------------------------+
  */
-/* $Id: exec.c,v 1.84.2.12 2003/08/28 15:54:35 sas Exp $ */
+/* $Id: exec.c,v 1.84.2.15.2.2 2005/10/26 20:03:14 tony2001 Exp $ */
 
 #include <stdio.h>
 #include "php.h"
@@ -398,7 +398,7 @@ PHP_FUNCTION(passthru)
 
    *NOT* safe for binary strings
 */
-char *php_escape_shell_cmd(char *str) {
+PHPAPI char *php_escape_shell_cmd(char *str) {
 	register int x, y, l;
 	char *cmd;
 	char *p = NULL;
@@ -410,6 +410,7 @@ char *php_escape_shell_cmd(char *str) {
 		switch (str[x]) {
 			case '"':
 			case '\'':
+#ifndef PHP_WIN32
 				if (!p && (p = memchr(str + x + 1, str[x], l - x - 1))) {
 					/* noop */
 				} else if (p && *p == str[x]) {
@@ -419,6 +420,7 @@ char *php_escape_shell_cmd(char *str) {
 				}
 				cmd[y++] = str[x];
 				break;
+#endif
 			case '#': /* This is character-set independent */
 			case '&':
 			case ';':
@@ -440,6 +442,12 @@ char *php_escape_shell_cmd(char *str) {
 			case '\\':
 			case '\x0A': /* excluding these two */
 			case '\xFF':
+#ifdef PHP_WIN32
+			/* since Windows does not allow us to escape these chars, just remove them */
+			case '%':
+				cmd[y++] = ' ';
+				break;
+#endif
 				cmd[y++] = '\\';
 				/* fall-through */
 			default:
@@ -454,7 +462,7 @@ char *php_escape_shell_cmd(char *str) {
 
 /* {{{ php_escape_shell_arg
  */
-char *php_escape_shell_arg(char *str) {
+PHPAPI char *php_escape_shell_arg(char *str) {
 	int x, y, l;
 	char *cmd;
 
@@ -462,21 +470,35 @@ char *php_escape_shell_arg(char *str) {
 	l = strlen(str);
 	
 	cmd = emalloc(4 * l + 3); /* worst case */
-	
+#ifdef PHP_WIN32
+	cmd[y++] = '"';
+#else
 	cmd[y++] = '\'';
+#endif
 	
 	for (x = 0; x < l; x++) {
 		switch (str[x]) {
+#ifdef PHP_WIN32
+		case '"':
+		case '%':
+			cmd[y++] = ' ';
+			break;
+#else
 		case '\'':
 			cmd[y++] = '\'';
 			cmd[y++] = '\\';
 			cmd[y++] = '\'';
+#endif
 			/* fall-through */
 		default:
 			cmd[y++] = str[x];
 		}
 	}
+#ifdef PHP_WIN32
+	cmd[y++] = '"';
+#else
 	cmd[y++] = '\'';
+#endif
 	cmd[y] = '\0';
 	return cmd;
 }
@@ -580,6 +602,7 @@ static void proc_open_rsrc_dtor(zend_rsrc_list_entry *rsrc TSRMLS_DC)
 	WaitForSingleObject(child, INFINITE);
 	GetExitCodeProcess(child, &wstatus);
 	FG(pclose_ret) = wstatus;
+	CloseHandle(child);
 #else
 # if HAVE_SYS_WAIT_H
 	int wstatus;
