@@ -21,7 +21,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: file.c,v 1.279.2.38 2003/10/10 01:38:01 iliaa Exp $ */
+/* $Id: file.c,v 1.279.2.70.2.2 2005/07/26 09:32:57 hyanantha Exp $ */
 
 /* Synced with php 3.0 revision 1.218 1999-06-16 [ssb] */
 
@@ -49,12 +49,11 @@
 #define O_RDONLY _O_RDONLY
 #include "win32/param.h"
 #include "win32/winutil.h"
-#elif defined(NETWARE) && !defined(NEW_LIBC)
-/*#include <ws2nlm.h>*/
-#include <sys/socket.h>
-#include "netware/param.h"
 #else
 #include <sys/param.h>
+#if HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
 #if defined(NETWARE) && defined(USE_WINSOCK)
 #include <novsock2.h>
 #else
@@ -73,8 +72,6 @@
 #if HAVE_PWD_H
 #ifdef PHP_WIN32
 #include "win32/pwd.h"
-#elif defined(NETWARE)
-#include "netware/pwd.h"
 #else
 #include <pwd.h>
 #endif
@@ -120,6 +117,13 @@ php_file_globals file_globals;
 #endif
 
 /* }}} */
+
+#define PHP_STREAM_TO_ZVAL(stream, arg) \
+	php_stream_from_zval_no_verify(stream, arg); \
+	if (stream == NULL) {	\
+		RETURN_FALSE;	\
+	}
+
 /* {{{ ZTS-stuff / Globals / Prototypes */
 
 /* sharing globals is *evil* */
@@ -172,7 +176,7 @@ PHP_MINIT_FUNCTION(file)
 #endif
 
 	REGISTER_INI_ENTRIES();
-	
+
 	REGISTER_LONG_CONSTANT("SEEK_SET", SEEK_SET, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("SEEK_CUR", SEEK_CUR, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("SEEK_END", SEEK_END, CONST_CS | CONST_PERSISTENT);
@@ -189,11 +193,11 @@ PHP_MINIT_FUNCTION(file)
 	REGISTER_LONG_CONSTANT("STREAM_NOTIFY_REDIRECTED",		PHP_STREAM_NOTIFY_REDIRECTED,		CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("STREAM_NOTIFY_PROGRESS",		PHP_STREAM_NOTIFY_PROGRESS,			CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("STREAM_NOTIFY_FAILURE",			PHP_STREAM_NOTIFY_FAILURE,			CONST_CS | CONST_PERSISTENT);
-	
+
 	REGISTER_LONG_CONSTANT("STREAM_NOTIFY_SEVERITY_INFO",	PHP_STREAM_NOTIFY_SEVERITY_INFO, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("STREAM_NOTIFY_SEVERITY_WARN",	PHP_STREAM_NOTIFY_SEVERITY_WARN, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("STREAM_NOTIFY_SEVERITY_ERR",	PHP_STREAM_NOTIFY_SEVERITY_ERR,  CONST_CS | CONST_PERSISTENT);
-	
+
 #ifdef HAVE_FNMATCH
 	REGISTER_LONG_CONSTANT("FNM_NOESCAPE", FNM_NOESCAPE, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("FNM_PATHNAME", FNM_PATHNAME, CONST_CS | CONST_PERSISTENT);
@@ -233,7 +237,7 @@ PHP_FUNCTION(flock)
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, arg1);
+	PHP_STREAM_TO_ZVAL(stream, arg1);
 
 	if (php_stream_cast(stream, PHP_STREAM_AS_FD, (void*)&fd, 1) == FAILURE)	{
 		RETURN_FALSE;
@@ -247,15 +251,20 @@ PHP_FUNCTION(flock)
 		RETURN_FALSE;
 	}
 
+	if (arg_count == 3) {
+		convert_to_long_ex(arg3);
+		Z_LVAL_PP(arg3) = 0;
+	}
+
 	/* flock_values contains all possible actions
 	   if (arg2 & 4) we won't block on the lock */
 	act = flock_values[act - 1] | (Z_LVAL_PP(arg2) & 4 ? LOCK_NB : 0);
 	if (flock(fd, act)) {
 		if (errno == EWOULDBLOCK && arg_count == 3) {
-			ZVAL_LONG(*arg3, 1);
+			Z_LVAL_PP(arg3) = 1;
 		} else {
 			RETURN_FALSE;
-		}	
+		}
 	}
 	RETURN_TRUE;
 }
@@ -384,7 +393,7 @@ PHP_FUNCTION(get_meta_tags)
 				/* For BC */
 				php_strtolower(name, strlen(name));
 				if (have_content) {
-					add_assoc_string(return_value, name, value, 0); 
+					add_assoc_string(return_value, name, value, 0);
 				} else {
 					add_assoc_string(return_value, name, empty_string, 0);
 				}
@@ -395,7 +404,7 @@ PHP_FUNCTION(get_meta_tags)
 			}
 
 			name = value = NULL;
-				
+
 			/* Reset all of our flags */
 			in_tag = looking_for_val = 0;
 			have_name = saw_name = 0;
@@ -433,7 +442,7 @@ PHP_FUNCTION(file_get_contents)
 		return;
 	}
 
-	stream = php_stream_open_wrapper(filename, "rb", 
+	stream = php_stream_open_wrapper(filename, "rb",
 			(use_include_path ? USE_PATH : 0) | ENFORCE_SAFE_MODE | REPORT_ERRORS,
 			NULL);
 	if (!stream) {
@@ -460,7 +469,7 @@ PHP_FUNCTION(file_get_contents)
 	}
 
 	php_stream_close(stream);
-	
+
 }
 /* }}} */
 
@@ -486,7 +495,7 @@ PHP_FUNCTION(file)
 		return;
 	}
 
-	stream = php_stream_open_wrapper(filename, "rb", 
+	stream = php_stream_open_wrapper(filename, "rb",
 			(use_include_path ? USE_PATH : 0) | ENFORCE_SAFE_MODE | REPORT_ERRORS,
 			NULL);
 	if (!stream) {
@@ -499,16 +508,16 @@ PHP_FUNCTION(file)
  	if ((target_len = php_stream_copy_to_mem(stream, &target_buf, PHP_STREAM_COPY_ALL, 0))) {
  		s = target_buf;
  		e = target_buf + target_len;
- 	
+
  		if (!(p = php_stream_locate_eol(stream, target_buf, target_len TSRMLS_CC))) {
  			p = e;
  			goto parse_eol;
   		}
-  		
+
  		if (stream->flags & PHP_STREAM_FLAG_EOL_MAC) {
   			eol_marker = '\r';
- 		}	
- 	
+ 		}
+
  		do {
  			p++;
 parse_eol:
@@ -521,7 +530,7 @@ parse_eol:
   			}
  			s = p;
  		} while ((p = memchr(p, eol_marker, (e-p))));
- 		
+
  		/* handle any left overs of files without new lines */
  		if (s != e) {
  			p = e;
@@ -531,7 +540,7 @@ parse_eol:
 
  	if (target_buf) {
  		efree(target_buf);
- 	}	
+ 	}
 	php_stream_close(stream);
 }
 /* }}} */
@@ -555,7 +564,7 @@ PHP_FUNCTION(tempnam)
 	if (php_check_open_basedir(Z_STRVAL_PP(arg1) TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
-	
+
 	d = estrndup(Z_STRVAL_PP(arg1), Z_STRLEN_PP(arg1));
 	strlcpy(p, Z_STRVAL_PP(arg2), sizeof(p));
 
@@ -574,7 +583,7 @@ PHP_FUNCTION(tempnam)
 PHP_NAMED_FUNCTION(php_if_tmpfile)
 {
 	php_stream *stream;
-	
+
 	if (ZEND_NUM_ARGS() != 0) {
 		WRONG_PARAM_COUNT;
 	}
@@ -601,14 +610,15 @@ PHP_FUNCTION(stream_get_meta_data)
 	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &arg1) == FAILURE) {
 		WRONG_PARAM_COUNT;
 	}
-	php_stream_from_zval(stream, arg1);
+	PHP_STREAM_TO_ZVAL(stream, arg1);
 
 	array_init(return_value);
-	
+
 	if (stream->wrapperdata) {
 		MAKE_STD_ZVAL(newval);
 		*newval = *(stream->wrapperdata);
 		zval_copy_ctor(newval);
+		INIT_PZVAL(newval);
 
 		add_assoc_zval(return_value, "wrapper_data", newval);
 	}
@@ -619,19 +629,19 @@ PHP_FUNCTION(stream_get_meta_data)
 
 	if (stream->filterhead) {
 		php_stream_filter *filter;
-		
+
 		MAKE_STD_ZVAL(newval);
 		array_init(newval);
-		
+
 		for (filter = stream->filterhead; filter != NULL; filter = filter->next) {
 			add_next_index_string(newval, (char *)filter->fops->label, 1);
 		}
 
 		add_assoc_zval(return_value, "filters", newval);
 	}
-	
+
 	add_assoc_long(return_value, "unread_bytes", stream->writepos - stream->readpos);
-	
+
 	if (php_stream_is(stream, PHP_STREAM_IS_SOCKET))	{
 		php_netstream_data_t *sock = PHP_NETSTREAM_DATA_FROM_STREAM(stream);
 
@@ -692,7 +702,7 @@ static int stream_array_from_fd_set(zval *stream_array, fd_set *fds TSRMLS_DC)
 
 	ALLOC_HASHTABLE(new_hash);
 	zend_hash_init(new_hash, 0, NULL, ZVAL_PTR_DTOR, 0);
-	
+
 	for (zend_hash_internal_pointer_reset(Z_ARRVAL_P(stream_array));
 		 zend_hash_get_current_data(Z_ARRVAL_P(stream_array), (void **) &elem) == SUCCESS;
 		 zend_hash_move_forward(Z_ARRVAL_P(stream_array))) {
@@ -701,7 +711,7 @@ static int stream_array_from_fd_set(zval *stream_array, fd_set *fds TSRMLS_DC)
 		if (stream == NULL)
 			continue;
 
-		/* get the fd 
+		/* get the fd
 		 * NB: Most other code will NOT use the PHP_STREAM_CAST_INTERNAL flag
 		 * when casting.  It is only used here so that the buffered data warning
 		 * is not displayed.
@@ -723,7 +733,7 @@ static int stream_array_from_fd_set(zval *stream_array, fd_set *fds TSRMLS_DC)
 
 	zend_hash_internal_pointer_reset(new_hash);
 	Z_ARRVAL_P(stream_array) = new_hash;
-	
+
 	return ret;
 }
 
@@ -739,7 +749,7 @@ static int stream_array_emulate_read_fd_set(zval *stream_array TSRMLS_DC)
 
 	ALLOC_HASHTABLE(new_hash);
 	zend_hash_init(new_hash, 0, NULL, ZVAL_PTR_DTOR, 0);
-	
+
 	for (zend_hash_internal_pointer_reset(Z_ARRVAL_P(stream_array));
 		 zend_hash_get_current_data(Z_ARRVAL_P(stream_array), (void **) &elem) == SUCCESS;
 		 zend_hash_move_forward(Z_ARRVAL_P(stream_array))) {
@@ -774,7 +784,7 @@ static int stream_array_emulate_read_fd_set(zval *stream_array TSRMLS_DC)
 		zend_hash_destroy(new_hash);
 		FREE_HASHTABLE(new_hash);
 	}
-	
+
 	return ret;
 }
 /* }}} */
@@ -831,7 +841,7 @@ PHP_FUNCTION(stream_select)
 			RETURN_LONG(retval);
 		}
 	}
-	
+
 	retval = select(max_fd+1, &rfds, &wfds, &efds, tv_p);
 
 	if (retval == -1) {
@@ -858,13 +868,13 @@ static void user_space_stream_notifier(php_stream_context *context, int notifyco
 	zval *ps[6];
 	zval **ptps[6];
 	int i;
-	
+
 	for (i = 0; i < 6; i++) {
 		INIT_ZVAL(zvs[i]);
 		ps[i] = &zvs[i];
 		ptps[i] = &ps[i];
 	}
-		
+
 	ZVAL_LONG(ps[0], notifycode);
 	ZVAL_LONG(ps[1], severity);
 	if (xmsg) {
@@ -896,10 +906,10 @@ static int parse_context_options(php_stream_context *context, zval *options)
 	HashPosition pos, opos;
 	zval **wval, **oval;
 	char *wkey, *okey;
-	int wkey_len, okey_len;
+	uint wkey_len, okey_len;
 	int ret = SUCCESS;
 	ulong num_key;
-	
+
 	zend_hash_internal_pointer_reset_ex(Z_ARRVAL_P(options), &pos);
 	while (SUCCESS == zend_hash_get_current_data_ex(Z_ARRVAL_P(options), (void**)&wval, &pos)) {
 		if (HASH_KEY_IS_STRING == zend_hash_get_current_key_ex(Z_ARRVAL_P(options), &wkey, &wkey_len, &num_key, 0, &pos)
@@ -929,7 +939,7 @@ static int parse_context_params(php_stream_context *context, zval *params)
 	zval **tmp;
 
 	if ((ret = zend_hash_find(Z_ARRVAL_P(params), "notification", sizeof("notification"), (void**)&tmp)) == SUCCESS) {
-		
+
 		if (context->notifier) {
 			php_stream_notification_free(context->notifier);
 			context->notifier = NULL;
@@ -989,10 +999,9 @@ PHP_FUNCTION(stream_context_get_options)
 	context = decode_context_param(zcontext TSRMLS_CC);
 	ZEND_VERIFY_RESOURCE(context);
 
-	array_init(return_value);
 	*return_value = *context->options;
 	zval_copy_ctor(return_value);
-		
+	INIT_PZVAL(return_value);
 }
 /* }}} */
 
@@ -1057,12 +1066,12 @@ PHP_FUNCTION(stream_context_create)
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|a", &params) == FAILURE) {
 		RETURN_FALSE;
 	}
-	
+
 	context = php_stream_context_alloc();
-	
+
 	if (params)
 		parse_context_options(context, params);
-	
+
 	ZEND_REGISTER_RESOURCE(return_value, context, le_stream_context);
 }
 /* }}} */
@@ -1081,8 +1090,8 @@ static void apply_filter_to_stream(int append, INTERNAL_FUNCTION_PARAMETERS)
 		RETURN_FALSE;
 	}
 
-	php_stream_from_zval(stream, &zstream);
-	
+	PHP_STREAM_TO_ZVAL(stream, &zstream);
+
 	filter = php_stream_filter_create(filtername, filterparams, filterparamslen, php_stream_is_persistent(stream) TSRMLS_CC);
 	if (filter == NULL)
 		RETURN_FALSE;
@@ -1130,7 +1139,7 @@ PHP_NAMED_FUNCTION(php_if_fopen)
 	if (zcontext) {
 		ZEND_FETCH_RESOURCE(context, php_stream_context*, &zcontext, -1, "stream-context", le_stream_context);
 	}
-	
+
 	stream = php_stream_open_wrapper_ex(filename, mode,
 				(use_include_path ? USE_PATH : 0) | ENFORCE_SAFE_MODE | REPORT_ERRORS,
 				NULL, context);
@@ -1157,7 +1166,7 @@ PHPAPI PHP_FUNCTION(fclose)
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, arg1);
+	PHP_STREAM_TO_ZVAL(stream, arg1);
 	if (!stream->is_persistent) {
 		zend_list_delete(stream->rsrc_id);
 	} else {
@@ -1177,15 +1186,23 @@ PHP_FUNCTION(popen)
 	zval **arg1, **arg2;
 	FILE *fp;
 	char *p, *tmp = NULL;
-	char *b, buf[1024];
+	char *b, *buf = 0;
 	php_stream *stream;
-	
+
 	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &arg1, &arg2) == FAILURE) {
 		WRONG_PARAM_COUNT;
 	}
 	convert_to_string_ex(arg1);
 	convert_to_string_ex(arg2);
 	p = estrndup(Z_STRVAL_PP(arg2), Z_STRLEN_PP(arg2));
+#ifndef PHP_WIN32
+	{
+		char *z = memchr(p, 'b', Z_STRLEN_PP(arg2));
+		if (z) {
+			memmove(p + (z - p), z + 1, Z_STRLEN_PP(arg2) - (z - p));
+		}
+	}
+#endif
 	if (PG(safe_mode)){
 		b = strchr(Z_STRVAL_PP(arg1), ' ');
 		if (!b) {
@@ -1200,10 +1217,11 @@ PHP_FUNCTION(popen)
 				b = NULL;
 			}
 		}
+		
 		if (b) {
-			snprintf(buf, sizeof(buf), "%s%s", PG(safe_mode_exec_dir), b);
+			spprintf(&buf, 0, "%s%s", PG(safe_mode_exec_dir), b);
 		} else {
-			snprintf(buf, sizeof(buf), "%s/%s", PG(safe_mode_exec_dir), Z_STRVAL_PP(arg1));
+			spprintf(&buf, 0, "%s/%s", PG(safe_mode_exec_dir), Z_STRVAL_PP(arg1));
 		}
 
 		tmp = php_escape_shell_cmd(buf);
@@ -1212,8 +1230,13 @@ PHP_FUNCTION(popen)
 
 		if (!fp) {
 			php_error_docref2(NULL TSRMLS_CC, buf, p, E_WARNING, "%s", strerror(errno));
+			efree(p);
+			efree(buf);
 			RETURN_FALSE;
 		}
+		
+		efree(buf);
+
 	} else {
 		fp = VCWD_POPEN(Z_STRVAL_PP(arg1), p);
 		if (!fp) {
@@ -1241,12 +1264,12 @@ PHP_FUNCTION(pclose)
 {
 	zval **arg1;
 	php_stream *stream;
-	
+
 	if (ARG_COUNT(ht) != 1 || zend_get_parameters_ex(1, &arg1) == FAILURE) {
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, arg1);
+	PHP_STREAM_TO_ZVAL(stream, arg1);
 
 	zend_list_delete(stream->rsrc_id);
 	RETURN_LONG(FG(pclose_ret));
@@ -1264,7 +1287,7 @@ PHPAPI PHP_FUNCTION(feof)
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, arg1);
+	PHP_STREAM_TO_ZVAL(stream, arg1);
 
 	if (php_stream_eof(stream)) {
 		RETURN_TRUE;
@@ -1286,7 +1309,7 @@ PHP_FUNCTION(stream_set_blocking)
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, arg1);
+	PHP_STREAM_TO_ZVAL(stream, arg1);
 
 	convert_to_long_ex(arg2);
 	block = Z_LVAL_PP(arg2);
@@ -1321,7 +1344,7 @@ PHP_FUNCTION(stream_set_timeout)
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, socket);
+	PHP_STREAM_TO_ZVAL(stream, socket);
 
 	convert_to_long_ex(seconds);
 	t.tv_sec = Z_LVAL_PP(seconds);
@@ -1358,7 +1381,7 @@ PHPAPI PHP_FUNCTION(fgets)
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, arg1);
+	PHP_STREAM_TO_ZVAL(stream, arg1);
 
 	if (argc == 1) {
 		/* ask streams to give us a buffer of an appropriate size */
@@ -1369,8 +1392,8 @@ PHPAPI PHP_FUNCTION(fgets)
 		convert_to_long_ex(arg2);
 		len = Z_LVAL_PP(arg2);
 
-		if (len < 0) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Length parameter may not be negative");
+		if (len <= 0) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Length parameter must be greater than 0.");
 			RETURN_FALSE;
 		}
 
@@ -1378,7 +1401,7 @@ PHPAPI PHP_FUNCTION(fgets)
 		if (php_stream_get_line(stream, buf, len, &line_len) == NULL)
 			goto exit_failed;
 	}
-	
+
 	if (PG(magic_quotes_runtime)) {
 		Z_STRVAL_P(return_value) = php_addslashes(buf, line_len, &Z_STRLEN_P(return_value), 1 TSRMLS_CC);
 		Z_TYPE_P(return_value) = IS_STRING;
@@ -1412,7 +1435,7 @@ PHPAPI PHP_FUNCTION(fgetc)
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, arg1);
+	PHP_STREAM_TO_ZVAL(stream, arg1);
 
 	buf = emalloc(2 * sizeof(char));
 
@@ -1462,12 +1485,12 @@ PHPAPI PHP_FUNCTION(fgetss)
 		break;
 	}
 
-	php_stream_from_zval(stream, fd);
+	PHP_STREAM_TO_ZVAL(stream, fd);
 
 	convert_to_long_ex(bytes);
 	len = Z_LVAL_PP(bytes);
-	if (len < 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Length parameter may not be negative");
+	if (len <= 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Length parameter must be greater than 0.");
 		RETURN_FALSE;
 	}
 
@@ -1580,7 +1603,7 @@ PHPAPI PHP_FUNCTION(fwrite)
 		break;
 	}
 
-	php_stream_from_zval(stream, arg1);
+	PHP_STREAM_TO_ZVAL(stream, arg1);
 
 	if (!arg3 && PG(magic_quotes_runtime)) {
 		buffer = estrndup(Z_STRVAL_PP(arg2), Z_STRLEN_PP(arg2));
@@ -1608,7 +1631,7 @@ PHPAPI PHP_FUNCTION(fflush)
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, arg1);
+	PHP_STREAM_TO_ZVAL(stream, arg1);
 
 	ret = php_stream_flush(stream);
 	if (ret) {
@@ -1638,9 +1661,9 @@ PHP_FUNCTION(stream_set_write_buffer)
 		/* NOTREACHED */
 		break;
 	}
-	
-	php_stream_from_zval(stream, arg1);
-	
+
+	PHP_STREAM_TO_ZVAL(stream, arg1);
+
 	convert_to_long_ex(arg2);
 	buff = Z_LVAL_PP(arg2);
 
@@ -1666,7 +1689,7 @@ PHPAPI PHP_FUNCTION(rewind)
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, arg1);
+	PHP_STREAM_TO_ZVAL(stream, arg1);
 
 	if (-1 == php_stream_rewind(stream)) {
 		RETURN_FALSE;
@@ -1687,7 +1710,7 @@ PHPAPI PHP_FUNCTION(ftell)
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, arg1);
+	PHP_STREAM_TO_ZVAL(stream, arg1);
 
 	ret = php_stream_tell(stream);
 	if (ret == -1)	{
@@ -1710,7 +1733,7 @@ PHPAPI PHP_FUNCTION(fseek)
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, arg1);
+	PHP_STREAM_TO_ZVAL(stream, arg1);
 
 	convert_to_long_ex(arg2);
 	if (argcount > 2) {
@@ -1790,7 +1813,7 @@ PHP_FUNCTION(readfile)
 	int size=0;
 	int use_include_path=0;
 	php_stream *stream;
-	
+
 	/* check args */
 	switch (ZEND_NUM_ARGS()) {
 	case 1:
@@ -1862,7 +1885,7 @@ PHPAPI PHP_FUNCTION(fpassthru)
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, arg1);
+	PHP_STREAM_TO_ZVAL(stream, arg1);
 
 	size = php_stream_passthru(stream);
 	RETURN_LONG(size);
@@ -1887,11 +1910,12 @@ PHP_FUNCTION(rename)
 	old_name = Z_STRVAL_PP(old_arg);
 	new_name = Z_STRVAL_PP(new_arg);
 
-	if (PG(safe_mode) &&(!php_checkuid(old_name, NULL, CHECKUID_CHECK_FILE_AND_DIR))) {
+	if (PG(safe_mode) && (!php_checkuid(old_name, NULL, CHECKUID_CHECK_FILE_AND_DIR) ||
+				!php_checkuid(new_name, NULL, CHECKUID_CHECK_FILE_AND_DIR))) {
 		RETURN_FALSE;
 	}
 
-	if (php_check_open_basedir(old_name TSRMLS_CC)) {
+	if (php_check_open_basedir(old_name TSRMLS_CC) || php_check_open_basedir(new_name TSRMLS_CC)) {
 		RETURN_FALSE;
 	}
 
@@ -1900,12 +1924,35 @@ PHP_FUNCTION(rename)
 	if (ret == -1) {
 #ifdef EXDEV
 		if (errno == EXDEV) {
-			if (php_copy_file(old_name, new_name TSRMLS_CC)	== SUCCESS) {
-				VCWD_UNLINK(old_name);
-				RETURN_TRUE;
+			struct stat sb;
+			if (php_copy_file(old_name, new_name TSRMLS_CC) == SUCCESS) {
+				if (VCWD_STAT(old_name, &sb) == 0) {
+#if !defined(TSRM_WIN32) && !defined(NETWARE)
+					if (VCWD_CHMOD(new_name, sb.st_mode)) {
+						if (errno == EPERM) {
+							php_error_docref2(NULL TSRMLS_CC, old_name, new_name, E_WARNING, "%s", strerror(errno));
+							VCWD_UNLINK(old_name);
+							RETURN_TRUE;
+						}
+						php_error_docref2(NULL TSRMLS_CC, old_name, new_name, E_WARNING, "%s", strerror(errno));
+						RETURN_FALSE;
+					}
+					if (VCWD_CHOWN(new_name, sb.st_uid, sb.st_gid)) {
+						if (errno == EPERM) {
+							php_error_docref2(NULL TSRMLS_CC, old_name, new_name, E_WARNING, "%s", strerror(errno));
+							VCWD_UNLINK(old_name);
+							RETURN_TRUE;
+						}
+						php_error_docref2(NULL TSRMLS_CC, old_name, new_name, E_WARNING, "%s", strerror(errno));
+						RETURN_FALSE;
+					}
+#endif
+					VCWD_UNLINK(old_name);
+					RETURN_TRUE;
+				}
 			}
 		}
-#endif	
+#endif
 		php_error_docref2(NULL TSRMLS_CC, old_name, new_name, E_WARNING, "%s", strerror(errno));
 		RETURN_FALSE;
 	}
@@ -1958,7 +2005,7 @@ PHP_NAMED_FUNCTION(php_if_ftruncate)
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, fp);
+	PHP_STREAM_TO_ZVAL(stream, fp);
 
 	convert_to_long_ex(size);
 
@@ -1983,7 +2030,7 @@ PHP_NAMED_FUNCTION(php_if_fstat)
 	 	*stat_size, *stat_atime, *stat_mtime, *stat_ctime, *stat_blksize, *stat_blocks;
 	php_stream *stream;
 	php_stream_statbuf stat_ssb;
-	
+
 	char *stat_sb_names[13]={"dev", "ino", "mode", "nlink", "uid", "gid", "rdev",
 				  "size", "atime", "mtime", "ctime", "blksize", "blocks"};
 
@@ -1991,7 +2038,7 @@ PHP_NAMED_FUNCTION(php_if_fstat)
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, fp);
+	PHP_STREAM_TO_ZVAL(stream, fp);
 
 	if (php_stream_stat(stream, &stat_ssb)) {
 		RETURN_FALSE;
@@ -2006,16 +2053,22 @@ PHP_NAMED_FUNCTION(php_if_fstat)
 	MAKE_LONG_ZVAL_INCREF(stat_uid, stat_ssb.sb.st_uid);
 	MAKE_LONG_ZVAL_INCREF(stat_gid, stat_ssb.sb.st_gid);
 #ifdef HAVE_ST_RDEV
-	MAKE_LONG_ZVAL_INCREF(stat_rdev, stat_ssb.sb.st_rdev); 
+	MAKE_LONG_ZVAL_INCREF(stat_rdev, stat_ssb.sb.st_rdev);
 #else
-	MAKE_LONG_ZVAL_INCREF(stat_rdev, -1); 
+	MAKE_LONG_ZVAL_INCREF(stat_rdev, -1);
 #endif
 	MAKE_LONG_ZVAL_INCREF(stat_size, stat_ssb.sb.st_size);
+#ifdef NETWARE
+	MAKE_LONG_ZVAL_INCREF(stat_atime, stat_ssb.sb.st_atime.tv_sec);
+	MAKE_LONG_ZVAL_INCREF(stat_mtime, stat_ssb.sb.st_mtime.tv_sec);
+	MAKE_LONG_ZVAL_INCREF(stat_ctime, stat_ssb.sb.st_ctime.tv_sec);
+#else
 	MAKE_LONG_ZVAL_INCREF(stat_atime, stat_ssb.sb.st_atime);
 	MAKE_LONG_ZVAL_INCREF(stat_mtime, stat_ssb.sb.st_mtime);
 	MAKE_LONG_ZVAL_INCREF(stat_ctime, stat_ssb.sb.st_ctime);
+#endif
 #ifdef HAVE_ST_BLKSIZE
-	MAKE_LONG_ZVAL_INCREF(stat_blksize, stat_ssb.sb.st_blksize); 
+	MAKE_LONG_ZVAL_INCREF(stat_blksize, stat_ssb.sb.st_blksize);
 #else
 	MAKE_LONG_ZVAL_INCREF(stat_blksize,-1);
 #endif
@@ -2091,15 +2144,65 @@ PHPAPI int php_copy_file(char *src, char *dest TSRMLS_DC)
 {
 	php_stream *srcstream = NULL, *deststream = NULL;
 	int ret = FAILURE;
+	php_stream_statbuf src_s, dest_s;
 
-	srcstream = php_stream_open_wrapper(src, "rb", 
+	switch (php_stream_stat_path(src, &src_s)) {
+		case -1:
+			/* non-statable stream */
+			goto safe_to_copy;
+			break;
+		case 0:
+			break;
+		default: /* failed to stat file, does not exist? */
+			return ret;
+	}
+	if (php_stream_stat_path(dest, &dest_s) != 0) {
+		goto safe_to_copy;
+	}
+	if (!src_s.sb.st_ino || !dest_s.sb.st_ino) {
+		goto no_stat;
+	}
+	if (src_s.sb.st_ino == dest_s.sb.st_ino && src_s.sb.st_dev == dest_s.sb.st_dev) {
+		return ret;
+	} else {
+		goto safe_to_copy;
+	}
+no_stat:
+	{
+		char *sp, *dp;
+		int res;
+		
+		if ((sp = expand_filepath(src, NULL TSRMLS_CC)) == NULL) {
+			return ret;
+		}
+	 	if ((dp = expand_filepath(dest, NULL TSRMLS_CC)) == NULL) {
+	 		efree(sp);
+	 		goto safe_to_copy;
+	 	}
+
+		res = 
+#ifndef PHP_WIN32	 	
+			!strcmp(sp, dp);
+#else
+			!strcasecmp(sp, dp);
+#endif	
+
+		efree(sp);
+		efree(dp);
+		if (res) {
+			return ret;
+		}
+	}
+safe_to_copy:
+
+	srcstream = php_stream_open_wrapper(src, "rb",
 				STREAM_DISABLE_OPEN_BASEDIR | REPORT_ERRORS,
 				NULL);
-	
-	if (!srcstream) 
+
+	if (!srcstream)
 		return ret;
 
-	deststream = php_stream_open_wrapper(dest, "wb", 
+	deststream = php_stream_open_wrapper(dest, "wb",
 				ENFORCE_SAFE_MODE | REPORT_ERRORS,
 				NULL);
 
@@ -2127,12 +2230,12 @@ PHPAPI PHP_FUNCTION(fread)
 		WRONG_PARAM_COUNT;
 	}
 
-	php_stream_from_zval(stream, arg1);
+	PHP_STREAM_TO_ZVAL(stream, arg1);
 
 	convert_to_long_ex(arg2);
 	len = Z_LVAL_PP(arg2);
-	if (len < 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Length parameter may not be negative");
+	if (len <= 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Length parameter must be greater than 0.");
 		RETURN_FALSE;
 	}
 
@@ -2143,26 +2246,97 @@ PHPAPI PHP_FUNCTION(fread)
 	Z_STRVAL_P(return_value)[Z_STRLEN_P(return_value)] = 0;
 
 	if (PG(magic_quotes_runtime)) {
-		Z_STRVAL_P(return_value) = php_addslashes(Z_STRVAL_P(return_value), 
+		Z_STRVAL_P(return_value) = php_addslashes(Z_STRVAL_P(return_value),
 				Z_STRLEN_P(return_value), &Z_STRLEN_P(return_value), 1 TSRMLS_CC);
 	}
 	Z_TYPE_P(return_value) = IS_STRING;
 }
 /* }}} */
 
+static char *_php_fgetcsv_find_enclosure(char *start, int len, char enclosure, int end)
+{
+	char *s=start, *p, *e=start+len;
+
+	while (e > s && (p = memchr(s, enclosure, (e - s)))) {
+		if (p > s && *(p - 1) == '\\') {	/* check escape characters */
+			int enc_cnt=0;
+			char *pp = p - 1;
+
+			while (pp >= s && *pp == '\\') {
+				enc_cnt++;
+				pp--;
+			}
+			if ((enc_cnt % 2)) {
+				s = p + 1;
+				continue;
+			}
+		}
+		if (end) {
+			int i = 0;
+			while (e > p && *p == enclosure) {
+				s = ++p;
+				i++;
+			}
+			if (!(i % 2)) {
+				continue;
+			} else {
+				p--;
+			}
+		}
+		return p;
+	}
+
+	return NULL;
+}
+
+static void _php_fgetcsv_trim_enclosed(char *buf2, int *buf2_len, char enclosure)
+{
+	if (memchr(buf2, enclosure, *buf2_len)) {
+		int esc = 0, enc_c = 0, pos = 0;
+		while (pos < *buf2_len) {
+			if (*(buf2 + pos) == '\\') {
+				esc = !esc;
+				enc_c = 0;
+			} else if (*(buf2 + pos) == enclosure) {
+				if (esc) {
+					esc = 0;
+				} else if (enc_c) {
+					enc_c = 0;
+					memmove(buf2 + pos, buf2 + pos + 1, *buf2_len - pos - 1);
+					(*buf2_len)--;
+					continue;
+				} else if (!esc) {
+					enc_c = 2;
+				}
+			} else {
+				if (enc_c == 2) {
+					memmove(buf2 + pos - 1, buf2 + pos, *buf2_len - pos);
+					(*buf2_len)--;
+					enc_c--;
+				}
+				esc = 0;
+			}
+			pos++;
+		}
+		if (enc_c && *(buf2 + pos - 1) == enclosure) {
+			(*buf2_len)--;
+		}
+	}
+	buf2[*buf2_len] = '\0';
+}
+
 /* {{{ proto array fgetcsv(resource fp, int length [, string delimiter [, string enclosure]])
    Get line from file pointer and parse for CSV fields */
 PHP_FUNCTION(fgetcsv)
 {
-	char *temp, *tptr, *bptr, *lineEnd;
 	char delimiter = ',';	/* allow this to be set as parameter */
 	char enclosure = '"';	/* allow this to be set as parameter */
 
 	/* first section exactly as php_fgetss */
 
 	zval **fd, **bytes, **p_delim, **p_enclosure;
-	int len, temp_len;
-	char *buf;
+	size_t len, buf_len;
+	char *buf, *p, *s, *e, *re, *buf2=NULL;
 	php_stream *stream;
 
 	switch(ZEND_NUM_ARGS()) {
@@ -2203,163 +2377,143 @@ PHP_FUNCTION(fgetcsv)
 
 	if (ZEND_NUM_ARGS() >= 4) {
 		convert_to_string_ex(p_enclosure);
-		/* Make sure that there is at least one character in string */
-		if (Z_STRLEN_PP(p_enclosure) < 1) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "enclosure must be a character");
-			RETURN_FALSE;
+		if (Z_STRLEN_PP(p_enclosure) < 1) { /* no enclosure */
+			enclosure = 0;
+		} else { /* use first character from string */
+			enclosure = Z_STRVAL_PP(p_enclosure)[0];
 		}
-		/* use first character from string */
-		enclosure = Z_STRVAL_PP(p_enclosure)[0];
 	}
 
-	php_stream_from_zval(stream, fd);
+	PHP_STREAM_TO_ZVAL(stream, fd);
 
 	convert_to_long_ex(bytes);
 	len = Z_LVAL_PP(bytes);
-	if (len < 0) {
+	if (Z_LVAL_PP(bytes) < 0) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Length parameter may not be negative");
 		RETURN_FALSE;
 	}
 
 	buf = emalloc(len + 1);
-	/* needed because recv/read/gzread doesnt set null char at end */
-	memset(buf, 0, len + 1);
 
-	if (php_stream_gets(stream, buf, len) == NULL) {
+	if (php_stream_get_line(stream, buf, len, &buf_len) == NULL) {
 		efree(buf);
 		RETURN_FALSE;
 	}
+	s = buf;
+	re = e = buf + buf_len;
 
-	/* Now into new section that parses buf for delimiter/enclosure fields */
+	/* strip leading spaces */
+	while (isspace((int)*(unsigned char *)s) && *s != delimiter && s < re) {
+		s++;
+	}
 
-	/* Strip trailing space from buf, saving end of line in case required for enclosure field */
+	/* strip trailing spaces */
+	while (--e >= s && (*e == '\n' || *e == '\r') && *e != delimiter);
+	e++;
 
-	lineEnd = emalloc(len + 1);
-	bptr = buf;
-	tptr = buf + strlen(buf) -1;
-	while ( isspace((int)*(unsigned char *)tptr) && (*tptr!=delimiter) && (tptr > bptr) ) tptr--;
-	tptr++;
-	strcpy(lineEnd, tptr);
-
-	/* add single space - makes it easier to parse trailing null field */
-	*tptr++ = ' ';
-	*tptr = 0;
-
-	/* reserve workspace for building each individual field */
-
-	temp_len = len;
-	temp = emalloc(temp_len + 1);	/* unlikely but possible! */
-	tptr = temp;
-
-	/* Initialize return array */
 	array_init(return_value);
 
-	/* Main loop to read CSV fields */
-	/* NB this routine will return a single null entry for a blank line */
+#define CSV_ADD_ENTRY(os, es, st) {	\
+	if (es - st) {	\
+		add_next_index_stringl(return_value, os, es - st, 1);	\
+	} else {	\
+		add_next_index_string(return_value, "", 1);	\
+	}	\
+}
 
-	do {
-		/* 1. Strip any leading space */
-		while(isspace((int)*(unsigned char *)bptr) && (*bptr!=delimiter)) bptr++;
-		/* 2. Read field, leaving bptr pointing at start of next field */
-		if (enclosure && *bptr == enclosure) {
-			bptr++;	/* move on to first character in field */
+	if (!(e - s)) {
+		CSV_ADD_ENTRY(s, e, s);
+		goto done;
+	}
 
-			/* 2A. handle enclosure delimited field */
-			while (*bptr) {
-				/* we need to determine if the enclosure is 'real' or is it escaped */
-				if (*(bptr - 1) == '\\') {
-					int escape_cnt = 0;
-					char *bptr_p = bptr - 2;
-				
-					while (bptr_p > buf && *bptr_p == '\\') {
-						escape_cnt++;
-						bptr_p--;
-					}
-					if (!(escape_cnt % 2)) {
-						goto normal_char;
-						continue;
-					}
-				}
-			
-				if (*bptr == enclosure) {
-					/* handle the enclosure */
-					if ( *(bptr+1) == enclosure) {
-					/* embedded enclosure */
-						*tptr++ = *bptr; bptr +=2;
-					} else {
-					/* must be end of string - skip to start of next field or end */
-						while ( (*bptr != delimiter) && *bptr ) bptr++;
-						if (*bptr == delimiter) bptr++;
-						*tptr=0;	/* terminate temporary string */
-						break;	/* .. from handling this field - resumes at 3. */
-					}
-				} else {
-normal_char:
-				/* normal character */
-					*tptr++ = *bptr++;
-
-					if (*bptr == 0) {       /* embedded line end? */
-						*(tptr-1)=0;            /* remove space character added on reading line */
-						strcat(temp, lineEnd);   /* add the embedded line end to the field */
-
-						/* read a new line from input, as at start of routine */
-						memset(buf, 0, len+1);
-
-						if (php_stream_gets(stream, buf, len) == NULL) {
-							/* we've got an unterminated enclosure, assign all the data
-							 * from the start of the enclosure to end of data to the last element
-							 */
-							if (temp_len > len) { 
-								break;
-							}
-							
-							efree(lineEnd); 
-							efree(temp); 
-							efree(buf);
-							zval_dtor(return_value);
-							RETURN_FALSE;
-						}
-
-						temp_len += len;
-						temp = erealloc(temp, temp_len+1);
-						bptr = buf;
-						tptr = buf + strlen(buf) -1;
-						while (isspace((int)*(unsigned char *)tptr) && (*tptr!=delimiter) && (tptr > bptr)) 
-							tptr--;
-						tptr++; 
-						strcpy(lineEnd, tptr);
-						*tptr++ = ' ';  
-						*tptr = 0;
-
-						tptr = temp;      /* reset temp pointer to end of field as read so far */
-						while (*tptr) 
-							tptr++;
-					}
-				}
-			}
-		} else {
-			/* 2B. Handle non-enclosure field */
-			while ((*bptr != delimiter) && *bptr) 
-				*tptr++ = *bptr++;
-			*tptr=0;	/* terminate temporary string */
-
-			if (strlen(temp)) {
-				tptr--;
-				while (isspace((int)*(unsigned char *)tptr) && (*tptr!=delimiter)) 
-					*tptr-- = 0;	/* strip any trailing spaces */
-			}
-			
-			if (*bptr == delimiter) 
-				bptr++;
+csv_start:
+	if (!enclosure || !(p = _php_fgetcsv_find_enclosure(s, (e - s), enclosure, 0))) {
+no_enclosure:
+		while ((p = memchr(s, delimiter, (e - s)))) {
+			CSV_ADD_ENTRY(s, p, s);
+			s = p + 1;
+		}
+	} else {
+		char *p2=NULL;
+		int buf2_len=0;
+enclosure:
+		/* handle complete fields before the enclosure */
+		while (s < p && (p2 = memchr(s, delimiter, (p - s)))) {
+			CSV_ADD_ENTRY(s, p2, s);
+			s = p2 + 1;
 		}
 
-		/* 3. Now pass our field back to php */
-		add_next_index_string(return_value, temp, 1);
-		tptr = temp;
-	} while (*bptr);
+		p++;
+		/* strip leading spaces */
+		while (isspace((int)*(unsigned char *)s) && *s != delimiter && s < re) {
+			s++;
+		}
+		if (*s != enclosure) {
+			if ((p = memchr(s, delimiter, (e - s)))) {
+				CSV_ADD_ENTRY(s, p, s);
+			        s = p + 1;
+			        goto csv_start;
+			} else {
+				goto no_enclosure;
+			}
+		}
+		s++;
 
-	efree(lineEnd);
-	efree(temp);
+		/* try to find end of enclosure */
+		while (!(p2 = _php_fgetcsv_find_enclosure(p, (e - p), enclosure, 1))) {
+			buf2 = erealloc(buf2, buf2_len + (re - p) + 1);
+			memcpy(buf2 + buf2_len, p, (re - p));
+			buf2_len += (re - p);
+
+			if (php_stream_get_line(stream, buf, len, &buf_len) == NULL) {
+				goto enclosure_done;
+			}
+			s = p = buf;
+			re = e = buf + buf_len;
+
+			/* strip trailing spaces */
+			while (--e >= s && (*e == '\n' || *e == '\r') && *e != delimiter);
+			e++;
+		}
+
+		/* end of enclosure found, now find the delimeter */
+		if ((p = memchr(p2, delimiter, (e - p2)))) {
+			p2 = s;
+			s = p + 1;
+
+			/* copy data to buffer */
+			buf2 = erealloc(buf2, buf2_len + (p - p2) + 1);
+			memcpy(buf2 + buf2_len, p2, (p - p2));
+			buf2_len += p - p2;
+
+			_php_fgetcsv_trim_enclosed(buf2, &buf2_len, enclosure);
+			CSV_ADD_ENTRY(buf2, buf2_len, 0);
+			buf2_len = 0;
+
+			if (!(p = _php_fgetcsv_find_enclosure(s, (e - s), enclosure, 0))) {
+				goto no_enclosure;
+			} else {
+				goto enclosure;
+			}
+		} else {
+			buf2 = erealloc(buf2, buf2_len + (e - s) + 1);
+			memcpy(buf2 + buf2_len, s, (e - s));
+			buf2_len += e - s;
+enclosure_done:
+			_php_fgetcsv_trim_enclosed(buf2, &buf2_len, enclosure);
+			CSV_ADD_ENTRY(buf2, buf2_len, 0);
+
+			goto done;
+		}
+	}
+
+	if (s < e || (s > buf && *(s - 1) == delimiter)) {
+		CSV_ADD_ENTRY(s, e, s);
+	}
+done:
+	if (buf2) {
+		efree(buf2);
+	}
 	efree(buf);
 }
 /* }}} */
@@ -2514,13 +2668,13 @@ PHP_FUNCTION(fnmatch)
 	int filename_len;
 	long flags=0;
 
-	if (zend_parse_parameters(argc TSRMLS_CC, "ss|l", 
-							  &pattern, &pattern_len, 
-							  &filename, &filename_len, 
-							  &flags) 
-		== FAILURE) 
+	if (zend_parse_parameters(argc TSRMLS_CC, "ss|l",
+							  &pattern, &pattern_len,
+							  &filename, &filename_len,
+							  &flags)
+		== FAILURE)
 		return;
-	
+
 	RETURN_BOOL( ! fnmatch( pattern, filename, flags ));
 }
 /* }}} */
