@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP version 4.0                                                      |
+   | PHP Version 4                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2001 The PHP Group                                |
+   | Copyright (c) 1997-2003 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,11 +12,11 @@
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
    +----------------------------------------------------------------------+
-   | Authors: Paul Panotzki - Bunyip Information Systems                  |
+   | Author: Paul Panotzki - Bunyip Information Systems                   |
    +----------------------------------------------------------------------+
  */
 
-/* $Id: microtime.c,v 1.29 2001/02/26 06:07:23 andi Exp $ */
+/* $Id: microtime.c,v 1.39.2.2.2.1 2005/07/25 13:08:32 hyanantha Exp $ */
 
 #include "php.h"
 
@@ -43,12 +43,13 @@
 
 #define NUL  '\0'
 #define MICRO_IN_SEC 1000000.00
+#define SEC_IN_MIN 60
 
 /* {{{ proto string microtime(void)
    Returns a string containing the current time in seconds and microseconds */
+#ifdef HAVE_GETTIMEOFDAY
 PHP_FUNCTION(microtime)
 {
-#ifdef HAVE_GETTIMEOFDAY
 	struct timeval tp;
 	long sec = 0L;
 	double msec = 0.0;
@@ -61,18 +62,18 @@ PHP_FUNCTION(microtime)
 		if (msec >= 1.0) msec -= (long) msec;
 		snprintf(ret, 100, "%.8f %ld", msec, sec);
 		RETVAL_STRING(ret,1);
-	} else
-#endif
+	} else {
 		RETURN_FALSE;
+	}
 }
+#endif
 /* }}} */
 
-
 /* {{{ proto array gettimeofday(void)
    Returns the current time as array */
+#ifdef HAVE_GETTIMEOFDAY
 PHP_FUNCTION(gettimeofday)
 {
-#ifdef HAVE_GETTIMEOFDAY
 	struct timeval tp;
 	struct timezone tz;
 	
@@ -82,13 +83,18 @@ PHP_FUNCTION(gettimeofday)
 		array_init(return_value);
 		add_assoc_long(return_value, "sec", tp.tv_sec);
 		add_assoc_long(return_value, "usec", tp.tv_usec);
+#ifdef PHP_WIN32
+		add_assoc_long(return_value, "minuteswest", tz.tz_minuteswest/SEC_IN_MIN);
+#else
 		add_assoc_long(return_value, "minuteswest", tz.tz_minuteswest);
+#endif			
 		add_assoc_long(return_value, "dsttime", tz.tz_dsttime);
 		return;
-	} else
-#endif
-	RETURN_FALSE;
+	} else {
+		RETURN_FALSE;
+	}
 }
+#endif
 /* }}} */
 
 #ifdef HAVE_GETRUSAGE
@@ -104,7 +110,7 @@ PHP_FUNCTION(getrusage)
 	if(ac == 1 &&
 		zend_get_parameters_ex(ac, &pwho) != FAILURE) {
 		convert_to_long_ex(pwho);
-		if((*pwho)->value.lval == 1)
+		if(Z_LVAL_PP(pwho) == 1)
 			who = RUSAGE_CHILDREN;
 	}
 
@@ -116,7 +122,7 @@ PHP_FUNCTION(getrusage)
 	array_init(return_value);
 #define PHP_RUSAGE_PARA(a) \
 		add_assoc_long(return_value, #a, usg.a)
-#ifndef _OSD_POSIX /* BS2000 has only a few fields in the rusage struct */
+#if !defined( _OSD_POSIX) && !defined(__BEOS__) /* BS2000 has only a few fields in the rusage struct */
 	PHP_RUSAGE_PARA(ru_oublock);
 	PHP_RUSAGE_PARA(ru_inblock);
 	PHP_RUSAGE_PARA(ru_msgsnd);
@@ -129,6 +135,7 @@ PHP_FUNCTION(getrusage)
 	PHP_RUSAGE_PARA(ru_nsignals);
 	PHP_RUSAGE_PARA(ru_nvcsw);
 	PHP_RUSAGE_PARA(ru_nivcsw);
+	PHP_RUSAGE_PARA(ru_nswap);
 #endif /*_OSD_POSIX*/
 	PHP_RUSAGE_PARA(ru_utime.tv_usec);
 	PHP_RUSAGE_PARA(ru_utime.tv_sec);
@@ -140,10 +147,11 @@ PHP_FUNCTION(getrusage)
 
 /* }}} */
 
-
 /*
  * Local variables:
  * tab-width: 4
  * c-basic-offset: 4
  * End:
+ * vim600: sw=4 ts=4 fdm=marker
+ * vim<600: sw=4 ts=4
  */
