@@ -15,7 +15,7 @@
    | Author: Jim Winstead <jimw@php.net>                                  |
    +----------------------------------------------------------------------+
  */
-/* $Id: url.c,v 1.58.2.9 2003/10/13 04:28:33 iliaa Exp $ */
+/* $Id: url.c,v 1.58.2.21.2.1 2005/08/16 14:12:13 iliaa Exp $ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -60,15 +60,16 @@ PHPAPI void php_url_free(php_url *theurl)
 
 /* {{{ php_replace_controlchars
  */
-PHPAPI char *php_replace_controlchars(char *str)
+PHPAPI char *php_replace_controlchars_ex(char *str, int len)
 {
 	unsigned char *s = (unsigned char *)str;
+	unsigned char *e = (unsigned char *)str + len;
 	
 	if (!str) {
 		return (NULL);
 	}
 	
-	while (*s) {
+	while (s < e) {
 	    
 		if (iscntrl(*s)) {
 			*s='_';
@@ -79,22 +80,50 @@ PHPAPI char *php_replace_controlchars(char *str)
 	return (str);
 } 
 /* }}} */
- 
+
+PHPAPI char *php_replace_controlchars(char *str)
+{
+	return php_replace_controlchars_ex(str, strlen(str));
+} 
+
+PHPAPI php_url *php_url_parse(char const *str)
+{
+	return php_url_parse_ex(str, strlen(str));
+} 
 
 /* {{{ php_url_parse
  */
-PHPAPI php_url *php_url_parse(char *str)
+PHPAPI php_url *php_url_parse_ex(char const *str, int length)
 {
-	int length = strlen(str);
-	char port_buf[5];
+	char port_buf[6];
 	php_url *ret = ecalloc(1, sizeof(php_url));
-	char *s, *e, *p, *pp, *ue;
+	const char *s, *e, *p, *pp, *ue;
 		
 	s = str;
 	ue = s + length;
 
 	/* parse scheme */
-	if ((e = strchr(s, ':')) && (e-s)) {
+	if ((e = memchr(s, ':', length)) && (e - s)) {
+		/* validate scheme */
+		p = s;
+		while (p < e) {
+			/* scheme = 1*[ lowalpha | digit | "+" | "-" | "." ] */
+			if (!isalpha(*p) && !isdigit(*p) && *p != '+' && *p != '.' && *p != '-') {
+				if (e + 1 < ue) {
+					goto parse_port;
+				} else {
+					goto just_path;
+				}
+			}
+			p++;
+		}
+	
+		if (*(e + 1) == '\0') { /* only scheme is available */
+			ret->scheme = estrndup(s, (e - s));
+			php_replace_controlchars_ex(ret->scheme, (e - s));
+			goto end;
+		}
+
 		/* 
 		 * certain schemas like mailto: and zlib: may not have any / after them
 		 * this check ensures we support those.
@@ -113,14 +142,14 @@ PHPAPI php_url *php_url_parse(char *str)
 			}
 			
 			ret->scheme = estrndup(s, (e-s));
-			php_replace_controlchars(ret->scheme);
+			php_replace_controlchars_ex(ret->scheme, (e - s));
 			
 			length -= ++e - s;
 			s = e;
 			goto just_path;
 		} else {
 			ret->scheme = estrndup(s, (e-s));
-			php_replace_controlchars(ret->scheme);
+			php_replace_controlchars_ex(ret->scheme, (e - s));
 		
 			if (*(e+2) == '/') {
 				s = e + 3;
@@ -130,8 +159,8 @@ PHPAPI php_url *php_url_parse(char *str)
 					}
 				}
 			} else {
-				s = e + 1;
 				if (!strncasecmp("file", ret->scheme, sizeof("file"))) {
+					s = e + 1;
 					goto nohost;
 				} else {
 					length -= ++e - s;
@@ -164,8 +193,10 @@ PHPAPI php_url *php_url_parse(char *str)
 	
 	e = ue;
 	
-	if (!(p = strchr(s, '/'))) {
-		if ((p = strchr(s, '?'))) {
+	if (!(p = memchr(s, '/', (ue - s)))) {
+		if ((p = memchr(s, '?', (ue - s)))) {
+			e = p;
+		} else if ((p = memchr(s, '#', (ue - s)))) {
 			e = p;
 		}
 	} else {
@@ -177,24 +208,35 @@ PHPAPI php_url *php_url_parse(char *str)
 		if ((pp = memchr(s, ':', (p-s)))) {
 			if ((pp-s) > 0) {
 				ret->user = estrndup(s, (pp-s));
-				php_replace_controlchars(ret->user);
+				php_replace_controlchars_ex(ret->user, (pp - s));
 			}	
 		
 			pp++;
 			if (p-pp > 0) {
 				ret->pass = estrndup(pp, (p-pp));
-				php_replace_controlchars(ret->pass);
+				php_replace_controlchars_ex(ret->pass, (p-pp));
 			}	
 		} else {
 			ret->user = estrndup(s, (p-s));
-			php_replace_controlchars(ret->user);
+			php_replace_controlchars_ex(ret->user, (p-s));
 		}
 		
 		s = p + 1;
 	}
 	
 	/* check for port */
-	if ((p = memchr(s, ':', (e-s)))) {
+	if (*s == '[' && *(e-1) == ']') {
+		/* Short circuit portscan
+		   we're dealing with an
+		   IPv6 embedded address */
+		p = s;
+	} else {
+		/* memchr is a GNU specific extension
+		   Emulate for wide compatability */
+		for(p = e; *p != ':' && p >= s; p--);
+	}
+
+	if (p >= s && *p == ':') {
 		if (!ret->port) {
 			p++;
 			if (e-p > 5) { /* port cannot be longer then 5 characters */
@@ -213,7 +255,7 @@ PHPAPI php_url *php_url_parse(char *str)
 	} else {
 		p = e;
 	}
-	
+
 	/* check if we have a valid host, if we don't reject the string as url */
 	if ((p-s) < 1) {
 		STR_FREE(ret->scheme);
@@ -224,7 +266,7 @@ PHPAPI php_url *php_url_parse(char *str)
 	}
 	
 	ret->host = estrndup(s, (p-s));
-	php_replace_controlchars(ret->host);
+	php_replace_controlchars_ex(ret->host, (p - s));
 	
 	if (e == ue) {
 		return ret;
@@ -234,7 +276,7 @@ PHPAPI php_url *php_url_parse(char *str)
 	
 	nohost:
 	
-	if ((p = strchr(s, '?'))) {
+	if ((p = memchr(s, '?', (ue - s)))) {
 		pp = strchr(s, '#');
 		
 		if (pp && pp < p) {
@@ -244,24 +286,24 @@ PHPAPI php_url *php_url_parse(char *str)
 	
 		if (p - s) {
 			ret->path = estrndup(s, (p-s));
-			php_replace_controlchars(ret->path);
+			php_replace_controlchars_ex(ret->path, (p - s));
 		}	
 	
 		if (pp) {
 			if (pp - ++p) { 
 				ret->query = estrndup(p, (pp-p));
-				php_replace_controlchars(ret->query);
+				php_replace_controlchars_ex(ret->query, (pp - p));
 			}
 			p = pp;
 			goto label_parse;
 		} else if (++p - ue) {
 			ret->query = estrndup(p, (ue-p));
-			php_replace_controlchars(ret->query);
+			php_replace_controlchars_ex(ret->query, (ue - p));
 		}
-	} else if ((p = strchr(s, '#'))) {
+	} else if ((p = memchr(s, '#', (ue - s)))) {
 		if (p - s) {
 			ret->path = estrndup(s, (p-s));
-			php_replace_controlchars(ret->path);
+			php_replace_controlchars_ex(ret->path, (p - s));
 		}	
 		
 		label_parse:
@@ -269,13 +311,13 @@ PHPAPI php_url *php_url_parse(char *str)
 		
 		if (ue - p) {
 			ret->fragment = estrndup(p, (ue-p));
-			php_replace_controlchars(ret->fragment);
+			php_replace_controlchars_ex(ret->fragment, (ue - p));
 		}	
 	} else {
 		ret->path = estrndup(s, (ue-s));
-		php_replace_controlchars(ret->path);
+		php_replace_controlchars_ex(ret->path, (ue - s));
 	}
-
+end:
 	return ret;
 }
 /* }}} */
@@ -292,7 +334,7 @@ PHP_FUNCTION(parse_url)
 		return;
 	}
 
-	resource = php_url_parse(str);
+	resource = php_url_parse_ex(str, str_len);
 	if (resource == NULL) {
 		php_error_docref1(NULL TSRMLS_CC, str, E_WARNING, "Unable to parse url");
 		RETURN_FALSE;
@@ -330,12 +372,12 @@ static int php_htoi(char *s)
 	int value;
 	int c;
 
-	c = s[0];
+	c = ((unsigned char *)s)[0];
 	if (isupper(c))
 		c = tolower(c);
 	value = (c >= '0' && c <= '9' ? c - '0' : c - 'a' + 10) * 16;
 
-	c = s[1];
+	c = ((unsigned char *)s)[1];
 	if (isupper(c))
 		c = tolower(c);
 	value += c >= '0' && c <= '9' ? c - '0' : c - 'a' + 10;
