@@ -16,7 +16,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: wddx.c,v 1.96.2.5.2.1 2004/07/13 13:15:30 iliaa Exp $ */
+/* $Id: wddx.c,v 1.96.2.6.2.1 2005/08/10 22:39:12 iliaa Exp $ */
 
 #include "php.h"
 #include "php_wddx.h"
@@ -979,10 +979,20 @@ static void php_wddx_pop_element(void *user_data, const XML_Char *name)
 						
 						/* Clean up class name var entry */
 						zval_ptr_dtor(&ent1->data);
-					} else
-						zend_hash_update(target_hash,
-										 ent1->varname, strlen(ent1->varname)+1,
-										 &ent1->data, sizeof(zval *), NULL);
+					} else {
+						long l;  
+						double d;
+				
+						switch (is_numeric_string(ent1->varname, strlen(ent1->varname), &l, &d, 0)) {
+							case IS_DOUBLE:
+								l = (long) d;
+							case IS_LONG:
+								zend_hash_index_update(target_hash, l, &ent1->data, sizeof(zval *), NULL);
+								break;
+							default:
+								zend_hash_update(target_hash,ent1->varname, strlen(ent1->varname)+1, &ent1->data, sizeof(zval *), NULL);
+						}
+					}
 					efree(ent1->varname);
 				} else	{
 					zend_hash_next_index_insert(target_hash,
