@@ -16,7 +16,7 @@
    | Streams work by Wez Furlong <wez@thebrainroom.com>                   |
    +----------------------------------------------------------------------+
  */
-/* $Id: network.c,v 1.83.2.20 2003/10/17 11:09:49 sas Exp $ */
+/* $Id: network.c,v 1.83.2.27.2.1 2005/07/26 13:51:34 hyanantha Exp $ */
 
 /*#define DEBUG_MAIN_NETWORK 1*/
 
@@ -29,13 +29,6 @@
 #include <winsock.h>
 #define O_RDONLY _O_RDONLY
 #include "win32/param.h"
-#elif defined(NETWARE)
-#ifdef NEW_LIBC
-#include <sys/timeval.h>
-#include <sys/param.h>
-#else
-#include "netware/time_nw.h"
-#endif
 #else
 #include <sys/param.h>
 #endif
@@ -63,15 +56,11 @@
 
 #if defined(NETWARE)
 #ifdef USE_WINSOCK
-/*#include <ws2nlm.h>*/
 #include <novsock2.h>
 #else
-/* New headers for socket stuff */
-#ifdef NEW_LIBC
 #include <netinet/in.h>
 #include <netdb.h>
 #include <sys/select.h>
-#endif
 #include <sys/socket.h>
 #endif
 #elif !defined(PHP_WIN32)
@@ -430,7 +419,7 @@ PHPAPI int php_connect_nonb_win32(SOCKET sockfd,
  * port, returns the created socket on success, else returns -1.
  * timeout gives timeout in seconds, 0 means blocking mode.
  */
-int php_hostconnect(const char *host, unsigned short port, int socktype, struct timeval *timeout TSRMLS_DC)
+PHPAPI int php_hostconnect(const char *host, unsigned short port, int socktype, struct timeval *timeout TSRMLS_DC)
 {	
 	int n, repeatto, s;
 	struct sockaddr **sal, **psal;
@@ -628,6 +617,22 @@ PHPAPI php_stream *_php_stream_sock_open_from_socket(int socket, const char *per
 	memset(sock, 0, sizeof(php_netstream_data_t));
 
 	sock->is_blocked = 1;
+
+#if !defined(PHP_WIN32) && (defined(O_NONBLOCK) || defined(O_NDELAY))
+	if (socket >= 0 && socket < 3) {
+		/* mini-hack: if we are opening stdin, stdout or stderr,
+		 * we need to check to see if they are currently in
+		 * blocking or non-blocking mode. */
+		int flags = fcntl(socket, F_GETFL);
+
+#ifdef O_NONBLOCK
+		sock->is_blocked = !(flags & O_NONBLOCK);
+#else
+		sock->is_blocked = !(flags & O_NDELAY);
+#endif
+	}
+#endif
+	
 	sock->timeout.tv_sec = FG(default_socket_timeout);
 	sock->timeout.tv_usec = 0;
 	sock->socket = socket;
@@ -688,8 +693,11 @@ PHPAPI php_stream *_php_stream_sock_open_unix(const char *path, int pathlen, con
 	
 	memcpy(unix_addr.sun_path, path, pathlen);
 
-	if (php_connect_nonb(socketd, (struct sockaddr *) &unix_addr, sizeof(unix_addr), timeout) == SOCK_CONN_ERR) 
+	if (php_connect_nonb(socketd, (struct sockaddr *) &unix_addr,
+			sizeof(unix_addr), timeout) == SOCK_CONN_ERR) {
+		closesocket(socketd);
 		return NULL;
+	}
 
 	stream = php_stream_sock_open_from_socket_rel(socketd, persistent_id);
 	if (stream == NULL)
@@ -730,6 +738,8 @@ PHPAPI int php_stream_sock_ssl_activate_with_method(php_stream *stream, int acti
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "php_stream_sock_ssl_activate_with_method: failed to create an SSL context");
 			return FAILURE;
 		}
+	
+		SSL_CTX_set_options(ctx, SSL_OP_ALL);
 
 		sock->ssl_handle = php_SSL_new_from_context(ctx, stream TSRMLS_CC);
 
@@ -830,6 +840,30 @@ static void php_ERR_error_string_n(int code, char *buf, size_t size)
 	}
 }
 
+/* it doesn't matter that we do some hash traversal here, since it is done only
+ * in an error condition arising from a network connection problem */
+static int is_http_stream_talking_to_iis(php_stream *stream TSRMLS_DC)
+{
+	if (stream->wrapperdata && stream->wrapper && strcmp(stream->wrapper->wops->label, "HTTP") == 0) {
+		/* the wrapperdata is an array zval containing the headers */
+		zval **tmp;
+
+#define SERVER_MICROSOFT_IIS	"Server: Microsoft-IIS"
+		
+		zend_hash_internal_pointer_reset(Z_ARRVAL_P(stream->wrapperdata));
+		while (SUCCESS == zend_hash_get_current_data(Z_ARRVAL_P(stream->wrapperdata), (void**)&tmp)) {
+
+			if (strncasecmp(Z_STRVAL_PP(tmp), SERVER_MICROSOFT_IIS, sizeof(SERVER_MICROSOFT_IIS)-1) == 0) {
+				return 1;
+			}
+			
+			zend_hash_move_forward(Z_ARRVAL_P(stream->wrapperdata));
+		}
+	}
+	return 0;
+}
+
+
 static int handle_ssl_error(php_stream *stream, int nr_bytes TSRMLS_DC)
 {
 	php_netstream_data_t *sock = (php_netstream_data_t*)stream->abstract;
@@ -854,8 +888,11 @@ static int handle_ssl_error(php_stream *stream, int nr_bytes TSRMLS_DC)
 		case SSL_ERROR_SYSCALL:
 			if (ERR_peek_error() == 0) {
 				if (nr_bytes == 0) {
-					php_error_docref(NULL TSRMLS_CC, E_WARNING,
-							"SSL: fatal protocol error");
+					if (!is_http_stream_talking_to_iis(stream TSRMLS_CC)) {
+						php_error_docref(NULL TSRMLS_CC, E_WARNING,
+								"SSL: fatal protocol error");
+					}
+					SSL_set_shutdown(sock->ssl_handle, SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
 					stream->eof = 1;
 					retry = 0;
 				} else {
@@ -1133,7 +1170,7 @@ static int php_sockop_set_option(php_stream *stream, int option, int value, void
 }
 
 /* private API; don't use in extensions */
-int _php_network_is_stream_alive(php_stream *stream)
+int _php_network_is_stream_alive(php_stream *stream TSRMLS_DC)
 {
 	php_netstream_data_t *sock = (php_netstream_data_t*)stream->abstract;
 	int alive = 1;
@@ -1141,7 +1178,7 @@ int _php_network_is_stream_alive(php_stream *stream)
 	fd_set rfds;
 	struct timeval tv = {0, 0};
 	char buf;
-
+	
 	/* logic: if the select call indicates that there is data to
 	 * be read, but a read returns 0 bytes of data, then the socket
 	 * has been closed.
