@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 4                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2002 The PHP Group                                |
+   | Copyright (c) 1997-2003 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -16,7 +16,7 @@
    |          Zeev Suraski <zeev@zend.com>                                |
    +----------------------------------------------------------------------+
  */
-/* $Id: php_variables.c,v 1.35.2.3 2002/08/01 02:21:00 yohgaki Exp $ */
+/* $Id: php_variables.c,v 1.45.2.13.2.4 2005/10/02 11:33:27 rrichards Exp $ */
 
 #include <stdio.h>
 #include "php.h"
@@ -28,6 +28,9 @@
 
 #include "zend_globals.h"
 
+/* for systems that need to override reading of environment variables */
+void _php_import_environment_variables(zval *array_ptr TSRMLS_DC);
+PHPAPI void (*php_import_environment_variables)(zval *array_ptr TSRMLS_DC) = _php_import_environment_variables;
 
 PHPAPI void php_register_variable(char *var, char *strval, zval *track_vars_array TSRMLS_DC)
 {
@@ -39,7 +42,8 @@ PHPAPI void php_register_variable(char *var, char *strval, zval *track_vars_arra
 PHPAPI void php_register_variable_safe(char *var, char *strval, int str_len, zval *track_vars_array TSRMLS_DC)
 {
 	zval new_entry;
-
+	assert(strval != NULL);
+	
 	/* Prepare value */
 	Z_STRLEN(new_entry) = str_len;
 	if (PG(magic_quotes_gpc)) {
@@ -59,20 +63,19 @@ PHPAPI void php_register_variable_ex(char *var, zval *val, pval *track_vars_arra
 	char *ip;		/* index pointer */
 	char *index;
 	int var_len, index_len;
-	zval *gpc_element, **gpc_element_p, **top_gpc_p=NULL;
+	zval *gpc_element, **gpc_element_p;
 	zend_bool is_array;
-	zend_bool free_index;
 	HashTable *symtable1=NULL;
-	HashTable *symtable2=NULL;
 
-	if (PG(register_globals)) {
-		symtable1 = EG(active_symbol_table);
-	}
+	assert(var != NULL);
+	
 	if (track_vars_array) {
-		if (symtable1) {
-			symtable2 = Z_ARRVAL_P(track_vars_array);
-		} else {
-			symtable1 = Z_ARRVAL_P(track_vars_array);
+		symtable1 = Z_ARRVAL_P(track_vars_array);
+	} else if (PG(register_globals)) {
+		symtable1 = EG(active_symbol_table);
+		/* GLOBALS hijack attempt, reject parameter */
+		if (!strncmp("GLOBALS", var, sizeof("GLOBALS")) || !strncmp("GLOBALS", var, sizeof("GLOBALS[")-1)) {
+			return;
 		}
 	}
 	if (!symtable1) {
@@ -100,6 +103,13 @@ PHPAPI void php_register_variable_ex(char *var, zval *val, pval *track_vars_arra
 		zval_dtor(val);
 		return;
 	}
+
+	/* GLOBALS hijack attempt, reject parameter */
+	if (symtable1 == EG(active_symbol_table) && !strcmp("GLOBALS", var)) {
+		zval_dtor(val);
+		return;
+	}
+
 	/* ensure that we don't have spaces or dots in the variable name (not binary safe) */
 	for (p=var; *p; p++) {
 		switch(*p) {
@@ -112,11 +122,35 @@ PHPAPI void php_register_variable_ex(char *var, zval *val, pval *track_vars_arra
 
 	index = var;
 	index_len = var_len;
-	free_index = 0;
 
 	while (1) {
 		if (is_array) {
-			char *escaped_index;
+			char *escaped_index = NULL, *index_s;
+			int new_idx_len = 0;
+
+			ip++;
+			index_s = ip;
+			if (isspace(*ip)) {
+				ip++;
+			}
+			if (*ip==']') {
+				index_s = NULL;
+			} else {
+				ip = strchr(ip, ']');
+				if (!ip) {
+					/* PHP variables cannot contain '[' in their names, so we replace the character with a '_' */
+					*(index_s - 1) = '_';
+					
+					index_len = var_len = 0;
+					if (index) {
+						index_len = var_len = strlen(index);
+					}
+					goto plain_var;
+					return;
+				}
+				*ip = 0;
+				new_idx_len = strlen(index_s);	
+			}
 
 			if (!index) {
 				MAKE_STD_ZVAL(gpc_element);
@@ -139,27 +173,11 @@ PHPAPI void php_register_variable_ex(char *var, zval *val, pval *track_vars_arra
 					efree(escaped_index);
 				}
 			}
-			if (!top_gpc_p) {
-				top_gpc_p = gpc_element_p;
-			}
 			symtable1 = Z_ARRVAL_PP(gpc_element_p);
 			/* ip pointed to the '[' character, now obtain the key */
-			index = ++ip;
-			index_len = 0;
-			if (*ip=='\n' || *ip=='\r' || *ip=='\t' || *ip==' ') {
-				ip++;
-			}
-			if (*ip==']') {
-				index = NULL;
-			} else {
-				ip = strchr(ip, ']');
-				if (!ip) {
-					php_error(E_WARNING, "Missing ] in %s variable", var);
-					return;
-				}
-				*ip = 0;
-				index_len = strlen(index);
-			}
+			index = index_s;
+			index_len = new_idx_len;
+
 			ip++;
 			if (*ip=='[') {
 				is_array = 1;
@@ -168,27 +186,38 @@ PHPAPI void php_register_variable_ex(char *var, zval *val, pval *track_vars_arra
 				is_array = 0;
 			}
 		} else {
+plain_var:
 			MAKE_STD_ZVAL(gpc_element);
 			gpc_element->value = val->value;
 			Z_TYPE_P(gpc_element) = Z_TYPE_P(val);
 			if (!index) {
 				zend_hash_next_index_insert(symtable1, &gpc_element, sizeof(zval *), (void **) &gpc_element_p);
 			} else {
-				zend_hash_update(symtable1, index, index_len+1, &gpc_element, sizeof(zval *), (void **) &gpc_element_p);
-			}
-			if (!top_gpc_p) {
-				top_gpc_p = gpc_element_p;
+				zval **tmp;
+			
+				if (PG(magic_quotes_gpc) && (index!=var)) {
+					char *escaped_index = php_addslashes(index, index_len, &index_len, 0 TSRMLS_CC);
+					
+					if (PG(http_globals)[TRACK_VARS_COOKIE] && symtable1 == Z_ARRVAL_P(PG(http_globals)[TRACK_VARS_COOKIE]) && 
+							zend_hash_find(symtable1, escaped_index, index_len+1, (void **) &tmp) != FAILURE) {
+						efree(escaped_index);
+						break;
+					}
+					
+					zend_hash_update(symtable1, escaped_index, index_len+1, &gpc_element, sizeof(zval *), (void **) &gpc_element_p);
+					efree(escaped_index);
+				} else {
+					if (PG(http_globals)[TRACK_VARS_COOKIE] && symtable1 == Z_ARRVAL_P(PG(http_globals)[TRACK_VARS_COOKIE]) && 
+							zend_hash_find(symtable1, index, index_len+1, (void **) &tmp) != FAILURE) {
+						break;
+					}
+				
+					zend_hash_update(symtable1, index, index_len+1, &gpc_element, sizeof(zval *), (void **) &gpc_element_p);
+				}
 			}
 			break;
 		}
 	}
-
-	if (top_gpc_p) {
-		if (symtable2) {
-			zend_hash_update(symtable2, var, var_len+1, top_gpc_p, sizeof(zval *), NULL);
-			(*top_gpc_p)->refcount++;
-		}	
-	}
 }
 
 
@@ -218,8 +247,7 @@ SAPI_API SAPI_POST_HANDLER_FUNC(php_std_post_handler)
 	}
 }
 
-
-void php_treat_data(int arg, char *str, zval* destArray TSRMLS_DC)
+SAPI_API SAPI_TREAT_DATA_FUNC(php_default_treat_data)
 {
 	char *res = NULL, *var, *val, *separator=NULL;
 	const char *c_var;
@@ -302,11 +330,10 @@ void php_treat_data(int arg, char *str, zval* destArray TSRMLS_DC)
 			php_url_decode(var, strlen(var));
 			val_len = php_url_decode(val, strlen(val));
 			php_register_variable_safe(var, val, val_len, array_ptr TSRMLS_CC);
-		}
-		else {
+		} else {
+			php_url_decode(var, strlen(var));
 			php_register_variable_safe(var, "", 0, array_ptr TSRMLS_CC);
 		}
-		
 		var = php_strtok_r(NULL, separator, &strtok_buf);
 	}
 
@@ -319,8 +346,7 @@ void php_treat_data(int arg, char *str, zval* destArray TSRMLS_DC)
 	}
 }
 
-
-void php_import_environment_variables(zval *array_ptr TSRMLS_DC)
+void _php_import_environment_variables(zval *array_ptr TSRMLS_DC)
 {
 	char **env, *p, *t;
 
