@@ -91,8 +91,8 @@
 #define PHP_MODE_STRIP       5
 #define PHP_MODE_CLI_DIRECT  6
 
-static char *optarg = NULL;
-static int optind = 1;
+static char *php_optarg = NULL;
+static int php_optind = 1;
 
 static const opt_struct OPTIONS[] = {
 	{'a', 0, "interactive"},
@@ -130,7 +130,7 @@ static int module_name_cmp(const void *a, const void *b TSRMLS_DC)
 	Bucket *f = *((Bucket **) a);
 	Bucket *s = *((Bucket **) b);
 
-	return strcmp(((zend_module_entry *)f->pData)->name,
+	return strcasecmp(((zend_module_entry *)f->pData)->name,
 				  ((zend_module_entry *)s->pData)->name);
 }
 
@@ -214,7 +214,10 @@ static int sapi_cli_ub_write(const char *str, uint str_length TSRMLS_DC)
 
 static void sapi_cli_flush(void *server_context)
 {
-	if (fflush(stdout)==EOF) {
+	/* Ignore EBADF here, it's caused by the fact that STDIN/STDOUT/STDERR streams
+	 * are/could be closed before fflush() is called.
+	 */
+	if (fflush(stdout)==EOF && errno!=EBADF) {
 		php_handle_aborted_connection();
 	}
 }
@@ -436,20 +439,23 @@ static void cli_register_file_handles(TSRMLS_D)
 	
 	ic.value = *zin;
 	ic.flags = CONST_CS;
-	ic.name = zend_strndup("STDIN", 6);
-	ic.name_len = 6;
+	ic.name = zend_strndup(ZEND_STRL("STDIN"));
+	ic.name_len = sizeof("STDIN");
+	ic.module_number = 0;
 	zend_register_constant(&ic TSRMLS_CC);
 
 	oc.value = *zout;
 	oc.flags = CONST_CS;
-	oc.name = zend_strndup("STDOUT", 7);
-	oc.name_len = 7;
+	oc.name = zend_strndup(ZEND_STRL("STDOUT"));
+	oc.name_len = sizeof("STDOUT");
+	oc.module_number = 0;
 	zend_register_constant(&oc TSRMLS_CC);
 
 	ec.value = *zerr;
 	ec.flags = CONST_CS;
-	ec.name = zend_strndup("STDERR", 7);
-	ec.name_len = 7;
+	ec.name = zend_strndup(ZEND_STRL("STDERR"));
+	ec.name_len = sizeof("STDERR");
+	ec.module_number = 0;
 	zend_register_constant(&ec TSRMLS_CC);
 
 	FREE_ZVAL(zin);
@@ -466,7 +472,7 @@ static int cli_seek_file_begin(zend_file_handle *file_handle, char *script_file,
 	*lineno = 1;
 
 	if (!(file_handle->handle.fp = VCWD_FOPEN(script_file, "rb"))) {
-		php_printf("Could not open input file: %s.\n", script_file);
+		php_printf("Could not open input file: %s\n", script_file);
 		return FAILURE;
 	}
 	file_handle->filename = script_file;
@@ -500,8 +506,8 @@ int main(int argc, char *argv[])
 	zend_file_handle file_handle;
 /* temporary locals */
 	int behavior=PHP_MODE_STANDARD;
-	int orig_optind=optind;
-	char *orig_optarg=optarg;
+	int orig_optind=php_optind;
+	char *orig_optarg=php_optarg;
 	char *arg_free=NULL, **arg_excp=&arg_free;
 	char *script_file=NULL;
 	zend_llist global_vars;
@@ -549,18 +555,18 @@ int main(int argc, char *argv[])
 #endif
 
 
-	while ((c = php_getopt(argc, argv, OPTIONS, &optarg, &optind, 0))!=-1) {
+	while ((c = php_getopt(argc, argv, OPTIONS, &php_optarg, &php_optind, 0))!=-1) {
 		switch (c) {
 		case 'c':
-			cli_sapi_module.php_ini_path_override = strdup(optarg);
+			cli_sapi_module.php_ini_path_override = strdup(php_optarg);
 			break;
 		case 'n':
 			cli_sapi_module.php_ini_ignore = 1;
 			break;
 		}
 	}
-	optind = orig_optind;
-	optarg = orig_optarg;
+	php_optind = orig_optind;
+	php_optarg = orig_optarg;
 
 	cli_sapi_module.executable_location = argv[0];
 
@@ -596,17 +602,29 @@ int main(int argc, char *argv[])
 			exit_status=1;
 			goto out_err;
 		}
-	
-		while ((c = php_getopt(argc, argv, OPTIONS, &optarg, &optind, 0)) != -1) {
+
+		/* here is the place for hard coded defaults which cannot be overwritten in the ini file */
+		INI_HARDCODED("register_argc_argv", "1");
+		INI_HARDCODED("html_errors", "0");
+		INI_HARDCODED("implicit_flush", "1");
+		INI_HARDCODED("output_buffering", "0");
+		INI_HARDCODED("max_execution_time", "0");
+
+		while ((c = php_getopt(argc, argv, OPTIONS, &php_optarg, &php_optind, 0)) != -1) {
 			switch (c) {
 
+			case 'd': /* define ini entries on command line */
+				define_command_line_ini_entry(php_optarg);
+				break;
+
 			case 'h': /* help & quit */
 			case '?':
 				php_output_startup();
 				php_output_activate(TSRMLS_C);
 				php_cli_usage(argv[0]);
 				php_end_ob_buffers(1 TSRMLS_CC);
-				exit_status=1;
+				exit_status=0;
+				zend_ini_deactivate(TSRMLS_C);
 				goto out_err;
 
 
@@ -616,7 +634,7 @@ int main(int argc, char *argv[])
 				}
 				php_print_info(0xFFFFFFFF TSRMLS_CC);
 				php_end_ob_buffers(1 TSRMLS_CC);
-				exit_status=1;
+				exit_status=0;
 				goto out;
 
 			case 'm': /* list compiled in modules */
@@ -628,7 +646,8 @@ int main(int argc, char *argv[])
 				print_extensions(TSRMLS_C);
 				php_printf("\n");
 				php_end_ob_buffers(1 TSRMLS_CC);
-				exit_status=1;
+				exit_status=0;
+				zend_ini_deactivate(TSRMLS_C);
 				goto out_err;
 
 			case 'v': /* show php version & quit */
@@ -636,12 +655,12 @@ int main(int argc, char *argv[])
 					goto err;
 				}
 #if ZEND_DEBUG
-				php_printf("PHP %s (%s) (built: %s %s) (DEBUG)\nCopyright (c) 1997-2003 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__, get_zend_version());
+				php_printf("PHP %s (%s) (built: %s %s) (DEBUG)\nCopyright (c) 1997-2004 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__, get_zend_version());
 #else
-				php_printf("PHP %s (%s) (built: %s %s)\nCopyright (c) 1997-2003 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__, get_zend_version());
+				php_printf("PHP %s (%s) (built: %s %s)\nCopyright (c) 1997-2004 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__, get_zend_version());
 #endif
 				php_end_ob_buffers(1 TSRMLS_CC);
-				exit_status=1;
+				exit_status=0;
 				goto out;
 
 			default:
@@ -651,15 +670,10 @@ int main(int argc, char *argv[])
 
         /* Set some CLI defaults */
 		SG(options) |= SAPI_OPTION_NO_CHDIR;
-		/* here is the place for hard coded defaults which cannot be overwritten in the ini file */
-		INI_HARDCODED("register_argc_argv", "1");
-		INI_HARDCODED("html_errors", "0");
-		INI_HARDCODED("implicit_flush", "1");
-		INI_HARDCODED("max_execution_time", "0");
 
-		optind = orig_optind;
-		optarg = orig_optarg;
-		while ((c = php_getopt(argc, argv, OPTIONS, &optarg, &optind, 0)) != -1) {
+		php_optind = orig_optind;
+		php_optarg = orig_optarg;
+		while ((c = php_getopt(argc, argv, OPTIONS, &php_optarg, &php_optind, 0)) != -1) {
 			switch (c) {
 
 			case 'a':	/* interactive mode */
@@ -670,9 +684,6 @@ int main(int argc, char *argv[])
 			case 'C': /* don't chdir to the script directory */
 				/* This is default so NOP */
 				break;
-			case 'd': /* define ini entries on command line */
-				define_command_line_ini_entry(optarg);
-				break;
 
 			case 'e': /* enable extended info output */
 				CG(extended_info) = 1;
@@ -683,12 +694,12 @@ int main(int argc, char *argv[])
 					param_error = "Either execute direct code or use a file.\n";
 					break;
 				}
-				script_file = optarg;
+				script_file = php_optarg;
 				break;
 
 			case 'g': /* define global variables on command line */
 				{
-					char *arg = estrdup(optarg);
+					char *arg = estrdup(php_optarg);
 
 					zend_llist_add_element(&global_vars, &arg);
 				}
@@ -721,7 +732,7 @@ int main(int argc, char *argv[])
 					break;
 				}
 				behavior=PHP_MODE_CLI_DIRECT;
-				exec_direct=optarg;
+				exec_direct=php_optarg;
 				break;
 
 			case 's': /* generate highlighted HTML from source */
@@ -741,7 +752,7 @@ int main(int argc, char *argv[])
 				break;
 
 			case 'z': /* load extension file */
-				zend_load_extension(optarg);
+				zend_load_extension(php_optarg);
 				break;
 			case 'H':
 				hide_argv = 1;
@@ -761,9 +772,9 @@ int main(int argc, char *argv[])
 		CG(interactive) = interactive;
 
 		/* only set script_file if not set already and not in direct mode and not at end of parameter list */
-		if (argc > optind && !script_file && behavior!=PHP_MODE_CLI_DIRECT && strcmp(argv[optind-1],"--")) {
-			script_file=argv[optind];
-			optind++;
+		if (argc > php_optind && !script_file && behavior!=PHP_MODE_CLI_DIRECT && strcmp(argv[php_optind-1],"--")) {
+			script_file=argv[php_optind];
+			php_optind++;
 		}
 		if (script_file) {
 			if (cli_seek_file_begin(&file_handle, script_file, &lineno TSRMLS_CC) != SUCCESS) {
@@ -782,12 +793,12 @@ int main(int argc, char *argv[])
 
 		/* before registering argv to module exchange the *new* argv[0] */
 		/* we can achieve this without allocating more memory */
-		SG(request_info).argc=argc-optind+1;
-		arg_excp = argv+optind-1;
-		arg_free = argv[optind-1];
+		SG(request_info).argc=argc-php_optind+1;
+		arg_excp = argv+php_optind-1;
+		arg_free = argv[php_optind-1];
 		SG(request_info).path_translated = file_handle.filename;
-		argv[optind-1] = file_handle.filename;
-		SG(request_info).argv=argv+optind-1;
+		argv[php_optind-1] = file_handle.filename;
+		SG(request_info).argv=argv+php_optind-1;
 
 		if (php_request_startup(TSRMLS_C)==FAILURE) {
 			*arg_excp = arg_free;
